local getopt = require('getopt')
local utils = require('utils')
local ac = require('ansicolors')
local os = require('os')
local count = 0
line = '------------------------------------------------------------------------------'
mod = "                  ELECTRA fob cloning SCRIPT lua - jareckib"
version = '                       version --- v1.1.9  29/07/2023'
desc = [[

   Cloning new ELECTRA tags to T5577 tags. This script modifies block 0 and 
   writes additional data to blocks 3 and 4. It keeps track of last ELECTRA ID 
   used -> option -c
   For copy directly from the original ELECTRA tag (option: -e)
   For copy from input EM410X ID -> option -s .If press Y and <enter>
   it  which writes a ID. Any other input char ---> exit the script.
 
]]
example = [[
-------------------------------------------------------------------------------

------------------- cloning ELECTRA tag from input ID to T5577 tag ------------
  
  script run electra -s 11AA22BB55

------------------continue cloning from last cloned ELECTRA ------------------- 
  
  script run electra -c
  
---------------------- start cloning ELECTRA from original TAG ---------------- 
  
  script run electra -e
  
-------------------------------------------------------------------------------


]]
usage = [[
  script run electra.lua [-e] [-h] [-c] [-s <EM410x ID HEX number>]
]]
arguments = [[
    -h      : this help
    -c      : continue cloning from last ID used
    -s      : ELECTRA - EM410x ID HEX number
    -e      : Read original ELECTRA fro Proxmark3 device
    ]]
--------------------------------------Path to logs files
local DEBUG = false
local ID_STATUS = os.getenv('HOME')..'/.proxmark3/logs/log_'..os.date('%Y%m%d')..'.txt'
-------------------------------------------A debug printout-function
local function dbg(args)
    if not DEBUG then return end
    if type(args) == 'table' then
        local i = 1
        while args[i] do
            dbg(args[i])
            i = i+1
        end
    else
        print('###', args)
    end
end
-------------------------------------------------when errors occur
local function oops(err)
    core.console('clear')
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    print(ac.red..'               ERROR:'..ac.reset.. err)
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    core.clearCommandBuffer()
    return nil, errr
end
-- --------------------------------------------------Usage help
local function help()
    core.console('clear')
    print(line)
    print(ac.cyan..mod..ac.reset)
    print(ac.cyan..version..ac.reset)
    print(ac.yellow..desc..ac.reset)
    print(ac.cyan..'  Usage'..ac.reset)
    print(usage)
    print(ac.cyan..'  Arguments'..ac.reset)
    print(arguments)
    print(ac.cyan..'  Example usage'..ac.reset)
    print(example)
end
------------------------------------ Exit message
local function exitMsg(msg)
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    print(msg)
    print()
end
---------------------------------search EM ID
local function id()
    local f = io.open(ID_STATUS, "r")
    for line in f:lines() do
        id = line:match"^%[%+%] EM 410x ID (%x+)$"
        if id then break end
    end
    if not id then return nil, "No ID found in file" end
    local  f= io.open(ID_STATUS, "w")
    f:write(id)
    f:close()
    local  f= io.open(ID_STATUS, "r")
    if f == nil then
        return nil, print(ac.red.."                  Something wrong"..ac.reset)
    end
    local t = f:read("*all")
    f:close()
    local cn_hi  = tonumber(t:sub(1, 2), 16)
    local cn_low = tonumber(t:sub(3, 10), 16)
	return cn_hi, cn_low
end
--------------------------------------no id
local function writenoid()
    local  f= io.open(ID_STATUS, "w")
    f:write(("FFFFFFFFFF"):format(cn_hi, cn_low))
    f:close()
end
---------------------------------------read file
local function readfile()
    local f = io.open(ID_STATUS, "r")
    if f == nil then
        return nil, print(("   Could not read file "..ac.yellow.."%s"..ac.reset):format(ID_STATUS))
    end
    local t = f:read("*all")
    f:close()
    local cn_hi  = tonumber(t:sub(1, 2), 16)
    local cn_low = tonumber(t:sub(3, 10), 16)
	return cn_hi, cn_low
end
----------------------------------------write file
local function writefile(cn_hi, cn_low)
    local f = io.open(ID_STATUS, "w+")
    if f == nil then
        return nil, print(("   Could not write file "..ac.yellow.."%s"..ac.reset):format(ID_STATUS))
    end
    f:write(("%02X%08X\n"):format(cn_hi, cn_low))
    f:close()
    print(('  Saved EM410x ID '..ac.green..'%02X%08X'..ac.reset..' to TXT file:`'):format(cn_hi, cn_low))
	print((ac.yellow..'  %s'..ac.reset..'`'):format(ID_STATUS))
    return true, 'Ok'
end
---------------------------------------- main
local function main(args)
    print( string.rep('--',39) )
    print( string.rep('--',39) )
    print()
    if #args == 0 then return help() end
    local shall_continue = false
    local id_original = false
    local startid = ''
	for o, a in getopt.getopt(args, 'cp:s:h:ep') do
        if o == 'h' then return help() end
        if o == 'c' then shall_continue = true end
        if o == 's' then startid = a end
        if o == 'e' then id_original = true end
    end
    -- if start, check -s
    if not shall_continue and not id_original then 
        if startid == nil then return oops('       empty EM410x ID string') end
        if #startid == 0 then return oops('       empty EM410x ID string') end
        if #startid ~= 10 then return oops(' EM410x ID wrong length. Must be 5 hex bytes') end
    end
    core.console('clear')
    print( string.rep('--',39) )
    print(ac.red..'                        DISABLE HINTS FOR LESS OUTPUT'..ac.reset)
    print( string.rep('--',39) )
    core.console('pref set hint --off')
    local hi  = tonumber(startid:sub(1, 2), 16)
    local low = tonumber(startid:sub(3, 10), 16)
	if shall_continue then
        hi, low = readfile()
        print( string.rep('--',39) )
        print( string.rep('--',39) )
		print('')
        print('             Continue ELECTRA cloning from last saved ID')
    elseif id_original then
        print( string.rep('--',39) )
        print( string.rep('--',39) )
		print('')
        print('          Place original ELECTRA TAG on the Proxmark3 coil for read ')
        print('')
        print( string.rep('--',39) )
        print( string.rep('--',39) )
        os.execute("PAUSE")
        print('')
        print('   Readed ELECTRA ID: ')
        core.console('lf em 410x read')
        hi, low = id()
    else
        --hi, low = readfile()
        print( string.rep('--',39) )
        print( string.rep('--',39) )
        print( )
		print('          Starting ELECTRA cloning from entered EM-ELECTRA ID')
    end
	local template =('EM410x ID '..ac.green..'%02X%08X'..ac.reset)
    for i = low, low + 100, 1 do
        print('')
        print( string.rep('--',39) )
        local msg = (template):format(hi, low)
        print( string.rep('--',39) )
        if count > 0 then
            print(('  ELECTRA tags created: '..ac.green..'%s'..ac.reset):format(count))
        end
		print(('  %s'..ac.reset..' ELECTRA cloning to T5577? -'..ac.yellow..' y'..ac.reset..' for yes or '..ac.yellow..'enter'..ac.reset..' for exit'):format(msg))
        print('  Before confirming the cloning operation, put a blank '..ac.blue..'T5577'..ac.reset..' tag on coil PM3 !!!')
    local ans = utils.input( ' ')
        if ans == 'y' then
            core.console('clear')
            print( string.rep('--',39) )
            core.console( ('lf em 410x clone --id %02X%08X'):format(hi, low) )
            core.console('lf t55 write -b 0 -d 00148080')
            core.console('lf t55 write -b 3 -d 7E1EAAAA')
            core.console('lf t55 write -b 4 -d AAAAAAAA')
            count = count+1
        else
            core.console('clear')
            print( string.rep('--',39) )
            print(ac.red..'                                USER ABORTED'..ac.reset)
            print( string.rep('--',39) )
            break
        end
    end
    writefile(hi, low)
	print('  Enabling hints again')
    core.console('pref set hint --on')
    print( string.rep('--',39) )
	if count > 0 then
        print( string.rep('--',39) )print(('  ELECTRA tags created: '..ac.green..'%s'..ac.reset):format(count))
    end
end
main(args)